@@ -1,7 +1,7 @@
 #
 # closepairs.R
 #
-#   $Revision: 1.39 $   $Date: 2017/06/05 10:31:58 $
+#   $Revision: 1.44 $   $Date: 2019/10/28 08:48:47 $
 #
 #  simply extract the r-close pairs from a dataset
 # 
@@ -14,7 +14,8 @@ closepairs <- function(X, rmax, ...) {
   
 closepairs.ppp <- function(X, rmax, twice=TRUE,
                            what=c("all", "indices", "ijd"),
-                           distinct=TRUE, neat=TRUE, 
+                           distinct=TRUE, neat=TRUE,
+                           periodic=FALSE,
                            ...) {
   verifyclass(X, "ppp")
   what <- match.arg(what)
@@ -26,6 +27,10 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
     warning("Obsolete argument 'ordered' has been replaced by 'twice'")
     twice <- ordered
   }
+  if(periodic && !is.rectangle(Window(X)))
+    warning("Periodic edge correction applied in non-rectangular window",
+            call.=FALSE)
+  
   npts <- npoints(X)
   null.answer <- switch(what,
                         all = {
@@ -50,19 +55,58 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
                         })
   if(npts == 0)
     return(null.answer)
-  # sort points by increasing x coordinate
-  oo <- fave.order(X$x)
-  Xsort <- X[oo]
-  # First make an OVERESTIMATE of the number of unordered pairs
-  nsize <- ceiling(2 * pi * (npts^2) * (rmax^2)/area(Window(X)))
-  nsize <- max(1024, nsize)
-  if(nsize > .Machine$integer.max) {
-    warning("Estimated number of close pairs exceeds maximum possible integer",
-            call.=FALSE)
-    nsize <- .Machine$integer.max
+  ## sort points by increasing x coordinate
+  if(!periodic) {
+    oo <- fave.order(X$x)
+    Xsort <- X[oo]
+  } 
+  ## First make an OVERESTIMATE of the number of unordered pairs
+  nsize <- list(...)$nsize # secret option to test memory overflow code
+  if(!is.null(nsize)) {
+    splat("Using nsize =", nsize)
+  } else {
+    #' normal usage 
+    nsize <- ceiling(2 * pi * (npts^2) * (rmax^2)/area(Window(X)))
+    nsize <- max(1024, nsize)
+    if(nsize > .Machine$integer.max) {
+     warning("Estimated number of close pairs exceeds maximum possible integer",
+              call.=FALSE)
+      nsize <- .Machine$integer.max
+    }
   }
-  # Now extract pairs
-  if(spatstat.options("closepairs.newcode")) {
+  ## Now extract pairs
+  if(periodic) {
+    ## special algorithm for periodic distance
+    got.twice <- TRUE
+    x <- X$x
+    y <- X$y
+    r <- rmax
+    p <- sidelengths(Frame(X))
+    ng <- nsize
+    storage.mode(x) <- "double"
+    storage.mode(y) <- "double"
+    storage.mode(r) <- "double"
+    storage.mode(p) <- "double"
+    storage.mode(ng) <- "integer"
+    z <- .Call("closePpair",
+               xx=x,
+               yy=y,
+               pp=p,
+               rr=r,
+               nguess=ng,
+               PACKAGE="spatstat")
+    i <- z[[1L]]
+    j <- z[[2L]]
+    d <- z[[3L]]
+    if(what == "all") {
+      xi <- x[i]
+      yi <- y[i]
+      xj <- x[j]
+      yj <- y[j]
+      dx <- xj - xi
+      dy <- yj - yi
+    }
+  } else if(spatstat.options("closepairs.newcode")) {
     # ------------------- use new faster code ---------------------
     # fast algorithms collect each distinct pair only once
     got.twice <- FALSE
@@ -112,6 +156,55 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
              d  <- z[[3L]]
            })
 
+  } else if(spatstat.options("closepairs.altcode")) {
+    #' experimental alternative code
+    got.twice <- FALSE
+    ng <- nsize
+    #
+    x <- Xsort$x
+    y <- Xsort$y
+    r <- rmax
+    storage.mode(x) <- "double"
+    storage.mode(y) <- "double"
+    storage.mode(r) <- "double"
+    storage.mode(ng) <- "integer"
+    switch(what,
+           all = {
+             z <- .Call("altVclosepairs",
+                        xx=x, yy=y, rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
+             if(length(z) != 9)
+               stop("Internal error: incorrect format returned from altVclosepairs")
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+             xi <- z[[3L]]
+             yi <- z[[4L]]
+             xj <- z[[5L]]
+             yj <- z[[6L]]
+             dx <- z[[7L]]
+             dy <- z[[8L]]
+             d  <- z[[9L]]
+           },
+           indices = {
+             z <- .Call("altVcloseIJpairs",
+                        xx=x, yy=y, rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
+             if(length(z) != 2)
+               stop("Internal error: incorrect format returned from altVcloseIJpairs")
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+           },
+           ijd = {
+             z <- .Call("altVcloseIJDpairs",
+                        xx=x, yy=y, rr=r, nguess=ng,
+                        PACKAGE = "spatstat")
+             if(length(z) != 3)
+               stop("Internal error: incorrect format returned from altVcloseIJDpairs")
+             i  <- z[[1L]]  # NB no increment required
+             j  <- z[[2L]]
+             d  <- z[[3L]]
+           })
+
   } else {
     # ------------------- use older code --------------------------
     if(!distinct) {
@@ -227,9 +320,12 @@ closepairs.ppp <- function(X, rmax, twice=TRUE,
     # ------------------- end code switch ------------------------
   }
   
-  # convert i,j indices to original sequence
-  i <- oo[i]
-  j <- oo[j]
+  if(!periodic) {
+    ## convert i,j indices to original sequence
+    i <- oo[i]
+    j <- oo[j]
+  }
+  
   if(twice) {
     ## both (i, j) and (j, i) should be returned
     if(!got.twice) {
@@ -606,7 +702,64 @@ crosspairquad <- function(Q, rmax, what=c("all", "indices")) {
   # convert all indices to serial numbers in union.quad(Q)
   # assumes data are listed first
   clXD$j <- npoints(X) + clXD$j
-  result <- list(rbind(as.data.frame(clX), as.data.frame(clXD)))
+  result <- as.list(rbind(as.data.frame(clX), as.data.frame(clXD)))
   return(result)
 }
 
+tweak.closepairs <- function(cl, rmax, i, deltax, deltay, deltaz) {
+  stopifnot(is.list(cl))
+  stopifnot(all(c("i", "j") %in% names(cl)))
+  if(!any(c("xi", "dx") %in% names(cl)))
+    stop("Insufficient data to update closepairs list")
+  check.1.real(rmax)
+  check.1.integer(i)
+  check.1.real(deltax)
+  check.1.real(deltay)
+  if("dz" %in% names(cl)) check.1.real(deltaz) else { deltaz <- NULL }
+  hit.i <- (cl$i == i)
+  hit.j <- (cl$j == i)
+  if(any(hit.i | hit.j)) {
+    mm <- hit.i & !hit.j
+    if(any(mm)) {
+      cl$xi[mm] <- cl$xi[mm] + deltax
+      cl$yi[mm] <- cl$yi[mm] + deltay
+      cl$dx[mm] <- cl$dx[mm] - deltax
+      cl$dy[mm] <- cl$dy[mm] - deltay
+      if(is.null(deltaz)) {
+        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2)
+      } else {
+        cl$zi[mm] <- cl$zi[mm] + deltaz
+        cl$dz[mm] <- cl$dz[mm] - deltaz
+        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2 + cl$dz[mm]^2)
+      }
+    }
+    mm <- hit.j & !hit.i
+    if(any(mm)) {
+      cl$xj[mm] <- cl$xj[mm] + deltax
+      cl$yj[mm] <- cl$yj[mm] + deltay
+      cl$dx[mm] <- cl$dx[mm] + deltax
+      cl$dy[mm] <- cl$dy[mm] + deltay
+      if(is.null(deltaz)) {
+        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2)
+      } else {
+        cl$zj[mm] <- cl$zj[mm] + deltaz
+        cl$dz[mm] <- cl$dz[mm] + deltaz
+        cl$d[mm] <- sqrt(cl$dx[mm]^2 + cl$dy[mm]^2 + cl$dz[mm]^2)
+      }
+    }
+    mm <- hit.i & hit.j
+    if(any(mm)) {
+      cl$xi[mm] <- cl$xi[mm] + deltax
+      cl$xj[mm] <- cl$xj[mm] + deltax
+      cl$yi[mm] <- cl$yi[mm] + deltay
+      cl$yj[mm] <- cl$yj[mm] + deltay
+      if(!is.null(deltaz)) {
+        cl$zi[mm] <- cl$zi[mm] + deltaz
+        cl$zj[mm] <- cl$zj[mm] + deltaz
+      }
+    }
+    if(any(lost <- (cl$d > rmax)))
+      cl <- as.list(as.data.frame(cl)[!lost, , drop=FALSE])
+  }
+  return(cl)
+}
