@@ -3,7 +3,7 @@
 #
 # flip x and y coordinates
 #
-# $Revision: 1.1 $ $Date: 2011/08/02 03:10:23 $ 
+# $Revision: 1.3 $ $Date: 2017/02/07 07:22:47 $ 
 #
 
 flipxy <- function(X) {
@@ -30,14 +30,16 @@ flipxy.owin <- function(X) {
   verifyclass(X, "owin")
   switch(X$type,
          rectangle={
-           W <- owin(X$yrange, X$xrange)
+           W <- owin(X$yrange, X$xrange, unitname=unitname(X))
          },
          polygonal={
            bdry <- lapply(X$bdry, flipxypolygon)
            W <- owin(poly=bdry, check=FALSE, unitname=unitname(X))
          },
          mask={
-           W <- owin(mask=t(X$m), xy=list(x=X$yrow, y=X$xcol))
+           W <- owin(mask=t(X$m),
+                     xy=list(x=X$yrow, y=X$xcol),
+                     unitname=unitname(X))
          },
          stop("Unrecognised window type")
          )
@@ -46,7 +48,7 @@ flipxy.owin <- function(X) {
 
 flipxy.psp <- function(X) {
   stopifnot(is.psp(X))
-  flipends <- (X$ends)[, c(2,1,4,3), drop=FALSE]
+  flipends <- (X$ends)[, c(2L,1L,4L,3L), drop=FALSE]
   as.psp(flipends, window=flipxy(X$window), marks=X$marks,
          unitname=unitname(X), check=FALSE)
 }
