@@ -1,7 +1,7 @@
 #
 # lpp.R
 #
-#  $Revision: 1.61 $   $Date: 2019/01/10 00:04:11 $
+#  $Revision: 1.69 $   $Date: 2020/03/18 06:05:22 $
 #
 # Class "lpp" of point patterns on linear networks
 
@@ -110,7 +110,7 @@ plot.lpp <- function(x, ..., main, add=FALSE,
   mx <- marks(x)
   if(use.marks && !is.null(dim(mx))) {
     implied.all <- is.null(which.marks)
-    want.several <- implied.all || !is.null(dim(mx <- mx[,which.marks]))
+    want.several <- implied.all || !is.null(dim(mx <- mx[,which.marks,drop=TRUE]))
     do.several <- want.several && !add && multiplot
     if(want.several)
       mx <- as.data.frame(mx) #' ditch hyperframe columns
@@ -128,9 +128,11 @@ plot.lpp <- function(x, ..., main, add=FALSE,
       if(do.plot) message("Plotting the first column of marks")
     }
   }
+  ## single plot
   ## determine space required, including legend
   P <- as.ppp(x)
-  a <- plot(P, ..., do.plot=FALSE)
+  a <- plot(P, ..., do.plot=FALSE,
+            use.marks=use.marks, which.marks=which.marks)
   if(!do.plot) return(a)
   ## initialise graphics space
   if(!add) {
@@ -152,6 +154,7 @@ plot.lpp <- function(x, ..., main, add=FALSE,
   ## plot points, legend, title
   ans <- do.call.matched(plot.ppp,
                          c(list(x=P, add=TRUE, main=main,
+                                use.marks=use.marks, which.marks=which.marks,
                                 show.all=show.all, show.window=FALSE),
                            list(...)),
                          extrargs=c("shape", "size", "pch", "cex",
@@ -241,7 +244,7 @@ print.summary.lpp <- function(x, ...) {
 }
 
 intensity.lpp <- function(X, ...) {
-  len <- sum(lengths.psp(as.psp(as.linnet(X))))
+  len <- sum(lengths_psp(as.psp(as.linnet(X))))
   if(is.multitype(X)) table(marks(X))/len else npoints(X)/len
 }
 
@@ -285,6 +288,11 @@ as.lpp <- function(x=NULL, y=NULL, seg=NULL, tp=NULL, ...,
       L <- as.linnet(L, sparse=sparse)
     if(is.ppp(x) && nomore) {
       X <- lpp(x, L)
+    } else if(is.data.frame(x) && nomore) {
+      X <- do.call(as.lpp,
+                   resolve.defaults(as.list(x),
+                                    list(...),
+                                    list(marks=marks, L=L, check=check)))
     } else if(is.null(x) && is.null(y) && !is.null(seg) && !is.null(tp)){
       X <- lpp(data.frame(seg=seg, tp=tp), L=L)
     } else {
@@ -419,6 +427,7 @@ local2lpp <- function(L, seg, tp, X=NULL, df.only=FALSE) {
   if(missing(j) || is.null(j))
     return(x)
   stopifnot(is.owin(j))
+  x <- repairNetwork(x)
   w <- j
   L <- x$domain
   if(is.vanilla(unitname(w)))
@@ -479,7 +488,11 @@ affine.lpp <- function(X,  mat=diag(c(1,1)), vec=c(0,0), ...) {
 shift.lpp <- function(X, vec=c(0,0), ..., origin=NULL) {
   verifyclass(X, "lpp")
   Y <- X
-  Y$domain <- shift(X$domain, vec=vec, ..., origin=origin)
+  Y$domain <- if(missing(vec)) {
+                shift(X$domain, ..., origin=origin)
+              } else {
+                shift(X$domain, vec=vec, ..., origin=origin)
+              }
   vec <- getlastshift(Y$domain)
   Y$data[, c("x","y")] <- shiftxy(X$data[, c("x","y")], vec=vec)
   # tack on shift vector
@@ -543,26 +556,6 @@ superimpose.lpp <- function(..., L=NULL) {
   return(Y)
 }
 
-#
-# interactive plot for lpp objects
-#
-
-iplot.lpp <- function(x, ..., xname) {
-  if(missing(xname))
-    xname <- short.deparse(substitute(x))
-  stopifnot(is.lpp(x))
-  ## predigest
-  L <- domain(x)
-  v <- vertices(L)
-  deg <- vertexdegree(L)
-  dv <- textstring(v, txt=paste(deg))
-  y <- layered(lines=as.psp(L),
-               vertices=v,
-               degree=dv,
-               points=as.ppp(x))
-  iplot(y, ..., xname=xname, visible=c(TRUE, FALSE, FALSE, TRUE))
-}
-
 identify.lpp <- function(x, ...) {
   verifyclass(x, "lpp")
   P <- as.ppp(x)
@@ -621,7 +614,7 @@ cut.lpp <- function(x, z=marks(x), ...) {
            marks(x) <- g
            return(x)
          },
-         list = stop("Don't know how to split according to a list"))
+         list = stop("Don't know how to cut according to a list"))
   stop("Format of z not understood")
 }
 
