@@ -3,10 +3,10 @@
 #
 #  signed/vector valued measures with atomic and diffuse components
 #
-#  $Revision: 1.71 $  $Date: 2017/08/31 08:27:12 $
+#  $Revision: 1.90 $  $Date: 2020/01/11 10:41:21 $
 #
 msr <- function(qscheme, discrete, density, check=TRUE) {
-  if(!inherits(qscheme, "quad"))
+  if(!is.quad(qscheme))
     stop("qscheme should be a quadrature scheme")
   nquad <- n.quad(qscheme)
   U <- union.quad(qscheme)
@@ -67,12 +67,22 @@ msr <- function(qscheme, discrete, density, check=TRUE) {
     colnames(discretepad) <- colnames(density)
   }
 
-  #
-  #
-  # Discretised measure (value of measure for each quadrature tile)
+  ##
+  ## Discretised measure (value of measure for each quadrature tile)
+  ## 
   val <- discretepad + wt * density
   if(is.matrix(density)) colnames(val) <- colnames(density)
-  #
+  ##
+  ##
+  
+  if(check && !all(ok <- complete.cases(val))) {
+    warning("Some infinite, NA or NaN increments were removed", call.=FALSE)
+    val         <- ok * val
+    discretepad <- ok * discretepad
+    density     <- ok * density
+  }
+  
+  ## finished
   out <- list(loc = U,
               val = val,
               atoms = Z,
@@ -83,6 +93,9 @@ msr <- function(qscheme, discrete, density, check=TRUE) {
   return(out)
 }
 
+weed.msr <- function(x) {
+}
+
 # Translation table for usage of measures
 #
 #           e.g. res <- residuals(fit, ...)
@@ -150,6 +163,8 @@ print.msr <- function(x, ...) {
   return(invisible(NULL))
 }
 
+summary.msr <- function(object, ...) { print(object) }
+
 is.multitype.msr <- function(X, ...) {
   is.multitype(X$loc, ...)
 }
@@ -200,42 +215,60 @@ split.msr <- function(x, f, drop=FALSE, ...) {
 
 integral.msr <- function(f, domain=NULL, ...) {
   stopifnot(inherits(f, "msr"))
-  if(!is.null(domain)) {
-    if (is.tess(domain)) 
-      return(sapply(tiles(domain), integral.msr, f = f))
-    f <- f[domain]
+  if(is.tess(domain)) {
+    result <- sapply(tiles(domain), integral.msr, f = f)
+    if(length(dim(result)) > 1) result <- t(result)
+    return(result)
   }
+  if(!is.null(domain)) 
+    f <- f[domain]
   y <- with(f, "increment")
-  if(is.matrix(y)) apply(y, 2, sum) else sum(y)
+  z <- if(is.matrix(y)) apply(y, 2, sum) else sum(y)
+  return(z)
 }
 
-augment.msr <- function(x, ..., sigma) {
+augment.msr <- function(x, ..., sigma, recompute=FALSE) {
   ## add a pixel image of the smoothed density component
   stopifnot(inherits(x, "msr"))
-  if(!is.null(attr(x, "smoothdensity"))) return(x)
+  if(!recompute && !is.null(attr(x, "smoothdensity"))) return(x)
   d <- ncol(as.matrix(x$val))
   xloc <- x$loc
   W <- as.owin(xloc)
-  if(missing(sigma)) sigma <- maxnndist(xloc, positive=TRUE)
-  if(is.multitype(xloc)) {
-    ## multitype case - split by type, smooth, sum
+  mt <- is.multitype(xloc)
+  if(missing(sigma)) {
+    sigma <- if(!mt) avenndist(xloc) else max(sapply(split(xloc), avenndist))
+    if(sigma == 0) sigma <- max(bw.scott(xloc))/5
+  }
+  if(mt) {
+    ## multitype case - split by type, extract smoothed part, then sum
     y <- lapply(split(x), augment.msr, sigma=sigma, ...)
     z <- lapply(y, attr, which="smoothdensity")
     if((nc <- ncol(x)) == 1) {
       ## scalar valued
-      smo <- Reduce("+", z)
+      smo <- im.apply(z, sum)
+      ## WAS:     z <- do.call(harmonise, unname(z))
+      ##          smo <- Reduce("+", z)
     } else {
       ## vector valued
       smo <- vector(mode="list", length=nc)
-      for(j in 1:nc) 
-        smo[[j]] <- Reduce("+", lapply(z, "[[", i=j))
+      for(j in 1:nc) {
+        zj <- lapply(z, "[[", i=j)
+        smo[[j]] <- im.apply(zj, sum)
+        ## WAS:    zj <- do.call(harmonise, unname(zj))
+        ##         smo[[j]] <- Reduce("+", zj)
+      }
       smo <- as.solist(smo)
     }
+    attr(smo, "sigma") <- sigma
     attr(x, "smoothdensity") <- smo
     return(x)
-  }   
-  ## smooth density unless constant
+  }
+  ## Single-type 
   xdensity <- as.matrix(x$density)
+  ## first weed out Inf, NA, NaN
+  if(!all(ok <- complete.cases(xdensity))) 
+    xdensity <- ok * xdensity
+  ## smooth density unless constant
   ra <- apply(xdensity, 2, range)
   varble <- apply(as.matrix(ra), 2, diff) > sqrt(.Machine$double.eps)
   ##
@@ -259,10 +292,20 @@ augment.msr <- function(x, ..., sigma) {
                              as.im, W=W)
     smo <- as.solist(smo)
   }
+  attr(smo, "sigma") <- sigma
   attr(x, "smoothdensity") <- smo
   return(x)
 }
 
+update.msr <- function(object, ...) {
+  #' reconcile internal data
+  if(!is.null(smo <- attr(object, "smoothdensity"))) {
+    sigma <- attr(smo, "sigma")
+    object <- augment.msr(object, ..., sigma=sigma, recompute=TRUE)
+  }
+  return(object)
+}
+
 plot.msr <- function(x, ..., add=FALSE,
                      how=c("image", "contour", "imagecontour"),
                      main=NULL, 
@@ -282,7 +325,8 @@ plot.msr <- function(x, ..., add=FALSE,
     if(is.matrix(x$discrete)) x$discrete <- rowSums(x$discrete)
     if(is.matrix(x$density))  x$density <- rowSums(x$density)
     if(!is.null(smo <- attr(x, "smoothdensity")) && inherits(smo, "solist"))
-      attr(x, "smoothdensity") <- Reduce("+", smo)
+      attr(x, "smoothdensity") <- im.apply(smo, sum, check=FALSE)
+      ## WAS: attr(x, "smoothdensity") <-  Reduce("+", smo)
   }
 
   d <- dim(x)[2]
@@ -303,7 +347,7 @@ plot.msr <- function(x, ..., add=FALSE,
     y <- split(x)
     typenames <- names(y)
     vecnames <- colnames(x$val)
-    y <- as.solist(Reduce(append, lapply(y, unstack)))
+    y <- unstack(y)
     names(y) <- as.vector(t(outer(typenames, vecnames, paste, sep=".")))
   } 
   #' ensure image of density is present
@@ -440,9 +484,9 @@ plot.msr <- function(x, ..., add=FALSE,
     loc   <- unmark(loci)
     id    <- marks(loci)
     # extract
-    valu  <- valu[id, ]
-    disc  <- disc[id, ]
-    dens  <- dens[id, ]
+    valu  <- valu[id, , drop=FALSE]
+    disc  <- disc[id, , drop=FALSE]
+    dens  <- dens[id, , drop=FALSE]
     wt    <- wt[id]
     atoms <- atoms[id]
   }
@@ -460,11 +504,11 @@ dim.msr <- function(x) { dim(as.matrix(x$val)) }
 
 dimnames.msr <- function(x) { list(NULL, colnames(x$val)) }
 
-smooth.msr <- function(X, ...) {
-  .Deprecated("Smooth.msr", package="spatstat",
-     msg="smooth.msr is deprecated: use the generic Smooth with a capital S")
-  Smooth(X, ...)
-}
+# smooth.msr <- function(X, ...) {
+#   .Deprecated("Smooth.msr", package="spatstat",
+#      msg="smooth.msr is deprecated: use the generic Smooth with a capital S")
+#   Smooth(X, ...)
+# }
 
 Smooth.msr <- function(X, ..., drop=TRUE) {
   verifyclass(X, "msr")
@@ -505,10 +549,46 @@ as.layered.msr <- local({
   as.layered.msr
 })
 
+unitname.msr <- function(x) unitname(x$loc)
+
+"unitname<-.msr" <- function(x, value) {
+  unitname(x$loc) <- value
+  return(x)
+}
 
 scalardilate.msr <- function(X, f, ...) {
-  X$loc <- Xloc <- scalardilate(X$loc, f, ...)
-  putlastshift(X, getlastshift(Xloc))
+  X$loc <- scalardilate(X$loc, f, ...)
+  X$density <- X$density/f^2
+  X$wt      <- X$wt * f^2
+  return(X)
+}
+
+rotate.msr <- function(X, angle=pi/2, ..., centre=NULL) {
+  X$loc <- rotate(X$loc, angle=angle, ..., centre=centre)
+  return(X)
+}
+
+flipxy.msr <- function(X) {
+  X$loc <- flipxy(X$loc)
+  return(X)
+}
+
+rescale.msr <- function(X, s, unitname) {
+  if(missing(unitname)) 
+    unitname <- NULL
+  if(missing(s) || is.null(s)) 
+    s <- 1/unitname(X)$multiplier
+  Y <- scalardilate(X, 1/s)
+  unitname(Y) <- rescale(unitname(X), s, unitname)
+  return(Y)
+}
+
+affine.msr <- function(X, mat = diag(c(1, 1)), vec = c(0, 0), ...) {
+  X$loc <- affine(X$loc, mat=mat, vec=vec, ...)
+  detmat <- abs(det(mat))
+  X$density <- X$density/detmat
+  X$wt      <- X$wt * detmat
+  return(X)
 }
 
 Ops.msr <- function(e1,e2=NULL){
@@ -522,6 +602,8 @@ Ops.msr <- function(e1,e2=NULL){
            call.=FALSE)
     e1 <- unclass(e1)
     e1[vn] <- lapply(e1[vn], .Generic)
+    if(!is.null(sm <- attr(e1, "smoothdensity")))
+      attr(e1, "smoothdensity") <- do.call(.Generic, sm)
     class(e1) <- "msr"
     return(e1)
   } else {
@@ -551,6 +633,22 @@ Ops.msr <- function(e1,e2=NULL){
       e1[vn] <- mapply(.Generic, e1[vn], e2[vn],
                        SIMPLIFY=FALSE)
       class(e1) <- "msr"
+      #' handle smoothed densities
+      sm1 <- attr(e1, "smoothdensity")
+      sm2 <- attr(e2, "smoothdensity")
+      sm <-
+        if(is.null(sm1) || is.null(sm2)) {
+          NULL
+        } else if(is.im(sm1) && is.im(sm2)) {
+          do.call(.Generic, list(sm1, sm2))
+        } else if(is.im(sm1) && is.solist(sm2)) {
+          mapply(.Generic, e2=sm2, MoreArgs=list(e1=sm1), SIMPLIFY=FALSE)
+        } else if(is.solist(sm1) && is.im(sm2)) {
+          mapply(.Generic, e1=sm1, MoreArgs=list(e2=sm2), SIMPLIFY=FALSE)
+        } else if(is.solist(sm1) && is.solist(sm2)) {
+          mapply(.Generic, e1=sm1, e2=sm2, SIMPLIFY=FALSE)
+        } else NULL
+      attr(e1, "smoothdensity") <- sm
       return(e1)
     } else if(m1 && is.numeric(e2)) {
       if(!is.element(.Generic, c("/", "*")))
@@ -561,6 +659,12 @@ Ops.msr <- function(e1,e2=NULL){
       e1 <- unclass(e1)
       e1[vn] <- lapply(e1[vn], .Generic, e2=e2)
       class(e1) <- "msr"
+      #' handle smoothed density
+      sm1 <- attr(e1, "smoothdensity")
+      sm <- if(is.null(sm1)) NULL else
+            if(is.im(sm1)) do.call(.Generic, list(e1=sm1, e2=e2)) else
+            if(is.solist(sm1)) solapply(sm1, .Generic, e2=e2) else NULL
+      attr(e1, "smoothdensity") <- sm
       return(e1)
     } else if(m2 && is.numeric(e1)) {
       if(.Generic != "*") 
@@ -571,6 +675,12 @@ Ops.msr <- function(e1,e2=NULL){
       e2 <- unclass(e2)
       e2[vn] <- lapply(e2[vn], .Generic, e1=e1)
       class(e2) <- "msr"
+      #' handle smoothed density
+      sm2 <- attr(e2, "smoothdensity")
+      sm <- if(is.null(sm2)) NULL else
+            if(is.im(sm2)) do.call(.Generic, list(e1=e1, e2=sm2)) else
+            if(is.solist(sm2)) solapply(sm2, .Generic, e1=e1) else NULL
+      attr(e2, "smoothdensity") <- sm
       return(e2)
     }
     stop(paste("Operation", sQuote(paste0("e1", .Generic, "e2")),
@@ -586,6 +696,7 @@ measurePositive <- function(x) {
   y$discrete <- pmax(0, x$discrete)
   y$density  <- pmax(0, x$density)
   y$val      <- y$discrete + y$wt * y$density
+  y <- update(y)
   return(y)
 }
 
@@ -596,6 +707,7 @@ measureNegative <- function(x) {
   y$discrete <- -pmin(0, x$discrete)
   y$density  <- -pmin(0, x$density)
   y$val      <- y$discrete + y$wt * y$density
+  y <- update(y)
   return(y)
 }
 
@@ -606,11 +718,32 @@ measureVariation <- function(x) {
   y$discrete <- abs(x$discrete)
   y$density  <- abs(x$density)
   y$val      <- y$discrete + y$wt * y$density
+  y <- update(y)
   return(y)
 }
 
 totalVariation <- function(x) integral(measureVariation(x))
-  
+
+measureDiscrete <- function(x) {
+  if(!inherits(x, "msr"))
+    stop("x must be a measure", call.=FALSE)
+  y <- x
+  y$density[]  <- 0
+  y$val        <- y$discrete
+  y <- update(y)
+  return(y)
+}
+
+measureContinuous <- function(x) {
+  if(!inherits(x, "msr"))
+    stop("x must be a measure", call.=FALSE)
+  y <- x
+  y$discrete[] <- 0
+  y$val        <- y$wt * y$density
+  y <- update(y)
+  return(y)
+}
+
 harmonise.msr <- local({
 
   harmonise.msr <- function(...) {
