@@ -1,7 +1,7 @@
 #
 # mppm.R
 #
-#  $Revision: 1.81 $   $Date: 2016/12/30 01:44:07 $
+#  $Revision: 1.95 $   $Date: 2020/03/11 05:29:58 $
 #
 
 mppm <- local({
@@ -11,6 +11,7 @@ mppm <- local({
 #%^!ifdef RANDOMEFFECTS                 
                    random=NULL,
 #%^!endif                 
+                   weights=NULL,
                    use.gam=FALSE,
 #%^!ifdef RANDOMEFFECTS                                    
                    reltol.pql=1e-3,
@@ -45,6 +46,13 @@ mppm <- local({
                  dQuote("interact"), 
                  "), or a hyperframe containing such objects", sep=""))
 
+    if(is.null(weights)) {
+      weights <- rep(1, npat)
+    } else {
+      check.nvector(weights, npat, things="rows of data", oneok=TRUE)
+      if(length(weights) == 1L) weights <- rep(weights, npat)
+    } 
+    
     backdoor <- list(...)$backdoor
     if(is.null(backdoor) || !is.logical(backdoor))
       backdoor <- FALSE
@@ -135,15 +143,17 @@ mppm <- local({
     Yclass <- data.sumry$classes[Yname]
     if(Yclass == "ppp") {
       ## convert to quadrature schemes, for efficiency's sake
-      Y <- solapply(Y, quadscheme)
-    } else {
-      if(Yclass != "quad")
-        stop(paste("Column", dQuote(Yname), "of data",
-                   "does not consist of point patterns (class ppp)",
-                   "nor of quadrature schemes (class quad)"))
+      Y <- solapply(Y, quadscheme, ...)
+      ## Ydescrip <- "point patterns" ## not used
+    } else if(Yclass == "quad") {
       Y <- as.solist(Y)
+      ## Ydescrip <- "quadrature schemes" ## not used
+    } else {
+      stop(paste("Column", dQuote(Yname), "of data",
+                 "does not consist of point patterns (class ppp)",
+                 "nor of quadrature schemes (class quad)"),
+           call.=FALSE)
     }
-  
     ## Extract sub-hyperframe of data named in formulae
     datanames <- names(data)
     used.cov.names <- allvars[allvars %in% datanames]
@@ -270,6 +280,8 @@ mppm <- local({
       ## now the nontrivial interaction terms
       for(j in (1:ninteract)[iused & !trivial]) {
         inter <- interaction[i,j,drop=TRUE]
+        if(!is.null(ss <- inter$selfstart)) 
+          interaction[i,j] <- inter <- ss(Yi$data, inter)
         prepj <- bt.frame(Yi, ~1, inter, ..., covariates=covariates,
                           allcovar=TRUE, use.gam=use.gam,
                           vnamebase=itags[j], vnameprefix=itags[j])
@@ -336,6 +348,9 @@ mppm <- local({
     }
     ## ---------------- E N D   o f    L O O P  --------------------------
     ##
+    ## add case weights
+    moadf$caseweight <- weights[moadf$id]
+    ##
     ## backdoor exit - Berman-Turner frame only - used by predict.mppm 
     if(backdoor)
       return(moadf)
@@ -403,16 +418,20 @@ mppm <- local({
     fmla <- as.formula(fmla)
     ## Fix scoping problem
     assign("glmmsubset", moadf$.mpl.SUBSET, envir=environment(fmla))
+    for(nama in colnames(moadf))
+      assign(nama,       moadf[[nama]],     envir=environment(fmla))
     ## Satisfy package checker
     glmmsubset <- .mpl.SUBSET <- moadf$.mpl.SUBSET
     .mpl.W      <- moadf$.mpl.W
-  
+    caseweight  <- moadf$caseweight
+    
     ## ---------------- FIT THE MODEL ------------------------------------
     want.trend <- prep0$info$want.trend
     if(want.trend && use.gam) {
       fitter <- "gam"
       ctrl <- do.call(gam.control, resolve.defaults(gcontrol, list(maxit=50)))
-      FIT  <- gam(fmla, family=quasi(link=log, variance=mu), weights=.mpl.W,
+      FIT  <- gam(fmla, family=quasi(link=log, variance=mu),
+                  weights=.mpl.W * caseweight,
                   data=moadf, subset=(.mpl.SUBSET=="TRUE"),
                   control=ctrl)
       deviants <- deviance(FIT)
@@ -423,7 +442,8 @@ mppm <- local({
       attr(fmla, "ctrl") <- ctrl # very strange way to pass argument
       fixed <- 42 # to satisfy package checker
       FIT  <- hackglmmPQL(fmla, random=random,
-                          family=quasi(link=log, variance=mu), weights=.mpl.W,
+                          family=quasi(link=log, variance=mu),
+                          weights=.mpl.W * caseweight,
                           data=moadf, subset=glmmsubset,
                           control=attr(fixed, "ctrl"),
                           reltol=reltol.pql)
@@ -432,13 +452,16 @@ mppm <- local({
     } else {
       fitter <- "glm"
       ctrl <- do.call(glm.control, resolve.defaults(gcontrol, list(maxit=50)))
-      FIT  <- glm(fmla, family=quasi(link="log", variance="mu"), weights=.mpl.W,
+      FIT  <- glm(fmla, family=quasi(link="log", variance="mu"),
+                  weights=.mpl.W * caseweight,
                   data=moadf, subset=(.mpl.SUBSET=="TRUE"),
                   control=ctrl)
       deviants <- deviance(FIT)
     }
+    env <- list2env(moadf, parent=sys.frame(sys.nframe()))
+    environment(FIT$terms) <- env
     ## maximised log-pseudolikelihood
-    W <- moadf$.mpl.W
+    W <- with(moadf, .mpl.W * caseweight)
     SUBSET <- moadf$.mpl.SUBSET
     Z <- (moadf$.mpl.Y != 0)
     maxlogpl <- -(deviants/2 + sum(log(W[Z & SUBSET])) + sum(Z & SUBSET))
@@ -469,7 +492,8 @@ mppm <- local({
                      fmla=fmla,
                      FIT=FIT,
                      moadf=moadf,
-                     Vnamelist=Vnamelist
+                     Vnamelist=Vnamelist,
+                     Isoffsetlist=Isoffsetlist
                      ),
                    Inter =
                    list(
@@ -525,6 +549,8 @@ mppm <- local({
 
   allpoisson <- function(x) all(sapply(x, is.poisson.interact))
 
+  marklevels <- function(x) { levels(marks(x)) }
+  
   errorInconsistentRows <- function(what, offending) {
     stop(paste("There are inconsistent",
                what,
@@ -657,3 +683,26 @@ simulate.mppm <- function(object, nsim=1, ..., verbose=TRUE) {
   }
   return(h)
 }
+
+model.matrix.mppm <- function(object, ..., keepNA=TRUE, separate=FALSE) {
+  FIT <- object$Fit$FIT
+  df <- object$Fit$moadf
+  environment(FIT) <- list2env(df)
+  if(keepNA) {
+    mm <- model.matrix(FIT, ..., subset=NULL, na.action=NULL)
+    if(nrow(mm) != nrow(df))
+      stop("Internal error: model matrix has wrong number of rows", call.=FALSE)
+  } else {
+    mm <- model.matrix(FIT, ...)
+    ok <- complete.cases(df) & df$.mpl.SUBSET
+    if(nrow(mm) != sum(ok))
+      stop("Internal error: model matrix has wrong number of rows", call.=FALSE)
+    df <- df[ok, , drop=FALSE]
+  }
+  if(separate) {
+    id <- df$id
+    mm <- split.data.frame(mm, id)  # see help(split)
+  }
+  return(mm)
+}
+
