@@ -2,7 +2,7 @@
 #
 #    multistrauss.S
 #
-#    $Revision: 2.21 $	$Date: 2014/04/30 07:57:47 $
+#    $Revision: 2.23 $	$Date: 2015/03/31 03:57:11 $
 #
 #    The multitype Strauss process
 #
@@ -100,6 +100,9 @@ MultiStrauss <- local({
        pot      = MSpotential,
        par      = list(types=NULL, radii = NULL), # to be filled in later
        parnames = c("possible types", "interaction distances"),
+       pardesc  = c("vector of possible types",
+                    "matrix of hardcore distances"),
+       hasInf   = FALSE,
        selfstart = function(X, self) {
          if(!is.null(self$par$types)) return(self)
          types <- levels(marks(X))
@@ -114,7 +117,7 @@ MultiStrauss <- local({
            if(length(types) == 0)
              stop(paste("The", sQuote("types"),"argument should be",
                         "either NULL or a vector of all possible types"))
-           if(any(is.na(types)))
+           if(anyNA(types))
              stop("NA's not allowed in types")
            if(is.factor(types)) {
              types <- levels(types)
@@ -127,13 +130,15 @@ MultiStrauss <- local({
        print = function(self) {
          radii <- self$par$radii
          types <- self$par$types
-         cat(paste(nrow(radii), "types of points\n"))
-         if(!is.null(types)) {
-           cat("Possible types: \n")
-           print(noquote(types))
-         } else cat("Possible types:\t not yet determined\n")
+         if(waxlyrical('gory')) {
+           splat(nrow(radii), "types of points")
+           if(!is.null(types)) {
+             splat("Possible types: ")
+             print(noquote(types))
+           } else splat("Possible types:\t not yet determined")
+         }
          cat("Interaction radii:\n")
-         print(self$par$radii)
+         print(signif(radii, getOption("digits")))
          invisible()
        },
        interpret = function(coeffs, self) {
@@ -156,7 +161,7 @@ MultiStrauss <- local({
          #
          return(list(param=list(gammas=gammas),
                      inames="interaction parameters gamma_ij",
-                     printable=round(gammas,4)))
+                     printable=dround(gammas)))
        },
        valid = function(coeffs, self) {
          # interaction parameters gamma[i,j]
@@ -220,11 +225,14 @@ MultiStrauss <- local({
       radii <- types
       types <- NULL
     }
+    radii[radii == 0] <- NA
     out <- instantiate.interact(BlankMSobject, list(types=types, radii = radii))
     if(!is.null(types))
       dimnames(out$par$radii) <- list(types, types)
     return(out)
   }
 
+  MultiStrauss <- intermaker(MultiStrauss, BlankMSobject)
+  
   MultiStrauss
 })
