@@ -4,7 +4,7 @@
 #	Class 'ppm' representing fitted point process models.
 #
 #
-#	$Revision: 2.104 $	$Date: 2014/04/16 02:54:35 $
+#	$Revision: 2.147 $	$Date: 2020/03/04 04:25:22 $
 #
 #       An object of class 'ppm' contains the following:
 #
@@ -58,27 +58,31 @@ function(x, ...,
   do.SE <- force.no.SE <- force.SE <- FALSE
   if(np == 0) {
     force.no.SE <- TRUE
+  } else if(!is.null(x$internal$VB)) {
+    force.no.SE <- TRUE
   } else if(!misswhat && ("se" %in% what)) {
     force.SE <- TRUE
   } else switch(spatstat.options("print.ppm.SE"),
                 always = { force.SE <- TRUE }, 
                 never  = { force.no.SE <- TRUE },
                 poisson = {
-                  do.SE <- is.poisson(x) &&
-                           !is.null(x$fitter) && (x$fitter != "gam") &&
-                           waxlyrical("extras", terselevel)
+                  do.SE <-
+                    is.poisson(x) &&
+                      !identical(x$fitter, "gam") &&
+                        (!is.null(x$varcov) || x$method != "logi") &&
+                          waxlyrical("extras", terselevel)
                 })
   do.SE <- (do.SE || force.SE) && !force.no.SE
 
   s <- summary.ppm(x, quick=if(do.SE) FALSE else "no variances")
         
   notrend <-    s$no.trend
-  stationary <- s$stationary
+#  stationary <- s$stationary
   poisson <-    s$poisson
   markeddata <- s$marked
   multitype  <- s$multitype
         
-  markedpoisson <- poisson && markeddata
+#  markedpoisson <- poisson && markeddata
   csr <- poisson && notrend && !markeddata
 
   special <- csr && all(c("model", "trend") %in% what)
@@ -94,43 +98,48 @@ function(x, ...,
         
       if(markeddata) mrk <- s$entries$marks
       if(multitype) {
-        cat("Possible marks:\n")
-        splat(paste(levels(mrk)))
+        splat(paste("Possible marks:",
+                    commasep(sQuote(levels(mrk)))))
         parbreak(terselevel)
       }
     }
     ## ----- trend --------------------------
     if("trend" %in% what) {
       if(!notrend) {
-        cat("Trend formula: ")
-        print(s$trend$formula, showEnv=FALSE)
+        splat("Log",
+              if(poisson) "intensity: " else "trend: ",
+              pasteFormula(s$trend$formula))
         parbreak(terselevel)
       }
 
-      tv <- s$trend$value
+      if(waxlyrical('space', terselevel) || !do.SE) {
+        ## print trend coefficients, unless redundant and space is tight
+        tv <- s$trend$value
       
-      if(length(tv) == 0) 
-        splat("[No trend coefficients]")
-      else {
-        thead <- paste0(s$trend$label, ":")
-        if(is.list(tv)) {
-          splat(thead)
-          for(i in seq_along(tv))
-            print(tv[[i]])
-        } else if(is.numeric(tv) && length(tv) == 1 && is.null(names(tv))) {
-          ## single number: append to end of current line
-          splat(paste0(thead, "\t", signif(tv, digits)))
-        } else {
-          ## some other format 
-          splat(thead)
-          print(tv)
+        if(length(tv) == 0) 
+          splat("[No trend coefficients]")
+        else {
+          thead <- paste0(s$trend$label, ":")
+          if(is.list(tv)) {
+            splat(thead)
+            for(i in seq_along(tv))
+              print(tv[[i]])
+          } else if(is.numeric(tv) && length(tv) == 1) {
+            ## single number: append to end of current line
+            tvn <- names(tv)
+            tveq <- if(is.null(tvn)) "\t" else paste(" ", tvn, "= ")
+            splat(paste0(thead, tveq, signif(tv, digits)))
+          } else {
+            ## some other format 
+            splat(thead)
+            print(tv)
+          }
         }
+        parbreak(terselevel)
       }
     }
 
-    parbreak(terselevel)
-
-    if(waxlyrical("extras", terselevel) &&
+    if(waxlyrical("space", terselevel) &&
        !is.null(cfa <- s$covfunargs) && length(cfa) > 0) {
       cfafitter <- s$cfafitter
       if(is.null(cfafitter)) {
@@ -156,8 +165,8 @@ function(x, ...,
 
   if("interaction" %in% what) {
     if(!poisson) {
-      print(s$interaction, family=FALSE,
-            brief=!waxlyrical("extras", terselevel))
+      print(s$interaction, family=FALSE, banner=FALSE, 
+            brief=!waxlyrical("extras"))
       parbreak(terselevel)
     }
   }
@@ -171,7 +180,8 @@ function(x, ...,
       splat("Standard errors unavailable; variance-covariance matrix is singular")
     } else if(!force.no.SE) {
       # standard error was voluntarily omitted
-      splat("For standard errors, type coef(summary(x))\n")
+      if(waxlyrical('space', terselevel))
+        splat("For standard errors, type coef(summary(x))\n")
     }
   }
   
@@ -194,20 +204,36 @@ function(x, ...,
     fitter <- s$fitter
     converged <- s$converged
     if(!is.null(fitter) && fitter %in% c("glm", "gam") && !converged)
-      splat(paste("*** Fitting algorithm for", sQuote(fitter),
-                "did not converge ***"))
+      splat("*** Fitting algorithm for", sQuote(fitter),
+            "did not converge ***")
   }
 
-  if(waxlyrical("extras", terselevel) && s$projected)
-    splat("Fit was projected to obtain a valid point process model")
+  if(waxlyrical("extras", terselevel) && s$projected) {
+    parbreak()
+    splat("Fit was emended to obtain a valid point process model")
+  }
+
+  if(identical(s$valid, FALSE) && waxlyrical("errors", terselevel)) {
+    parbreak()
+    splat("*** Model is not valid ***")
+    if(!all(is.finite(s$entries$coef))) {
+      splat("*** Some coefficients are NA or Inf ***")
+    } else {
+      splat("*** Interaction parameters are outside valid range ***")
+    }
+  } else if(is.na(s$valid) && waxlyrical("extras", terselevel)) {
+    parbreak()
+    splat("[Validity of model could not be checked]")
+  }
   
   return(invisible(NULL))
 }
 
 quad.ppm <- function(object, drop=FALSE, clip=FALSE) {
   if(!is.ppm(object)) {
-    if(inherits(object, "kppm")) object <- object$po else
-    stop("object is not of class ppm or kppm")
+    if(is.kppm(object)) object <- object$po else
+    if(is.lppm(object)) object <- object$fit else
+    stop("object is not of class ppm, kppm or lppm")
   }
   Q <- object$Q
   if(is.null(Q))
@@ -239,6 +265,9 @@ coef.ppm <- function(object, ...) {
   object$coef
 }
 
+hasglmfit <- function(object) {
+  return(!is.null(object$internal$glmfit))
+}
 
 getglmfit <- function(object) {
   verifyclass(object, "ppm")
@@ -282,6 +311,7 @@ getppmdatasubset <- function(object) {
   return(sub)
 }
 
+
 getppmOriginalCovariates <- function(object) {
   df <- as.data.frame(as.ppp(quad.ppm(object)))
   cova <- object$covariates
@@ -297,7 +327,11 @@ getppmOriginalCovariates <- function(object) {
   
 # ??? method for 'effects' ???
 
-valid.ppm <- function(object) {
+valid <- function(object, ...) {
+  UseMethod("valid")
+}
+
+valid.ppm <- function(object, warn=TRUE, ...) {
   verifyclass(object, "ppm")
   coeffs <- coef(object)
   # ensure all coefficients are fitted, and finite
@@ -305,7 +339,7 @@ valid.ppm <- function(object) {
     return(FALSE)
   # inspect interaction
   inte <- object$interaction
-  if(is.null(inte))
+  if(is.poisson(object))
     return(TRUE) # Poisson process
   # extract fitted interaction coefficients
   Vnames <- object$internal$Vnames
@@ -314,14 +348,21 @@ valid.ppm <- function(object) {
   # check interaction
   checker <- inte$valid
   if(is.null(checker) || !newstyle.coeff.handling(inte)) {
-    warning("Internal error: unable to check validity of model")
+    if(warn) warning("Internal error: unable to check validity of model")
     return(NA)
   }
+  #' remove prefix to obtain coefficient names expected by interaction
+  if(npre <- sum(nchar(object$internal$vnameprefix)))
+    names(Icoeffs) <- substring(names(Icoeffs), npre+1L)
   answer <- checker(Icoeffs, inte)
   return(answer)
 }
 
-project.ppm <- local({
+emend <- function(object, ...) {
+  UseMethod("emend")
+}
+
+emend.ppm <- project.ppm <- local({
   tracemessage <- function(depth, ...) {
     if(depth == 0) return(NULL)
     spacer <- paste(rep.int("  ", depth), collapse="")
@@ -332,7 +373,7 @@ project.ppm <- local({
   leaving <- function(depth) {
     tracemessage(depth, ngettext(depth, "Returning.", "Exiting level."))
   }
-  project.ppm <- function(object, ..., fatal=FALSE, trace=FALSE) {
+  emend.ppm <- function(object, ..., fatal=FALSE, trace=FALSE) {
     verifyclass(object, "ppm")
     fast <- spatstat.options("project.fast")
     # user specifies 'trace' as logical
@@ -381,7 +422,7 @@ project.ppm <- local({
           print(newobject)
         }
         # recurse
-        newobject <- project.ppm(newobject, fatal=fatal, trace=tdnext)
+        newobject <- emend.ppm(newobject, fatal=fatal, trace=tdnext)
         # return
         leaving(td)
         return(newobject)
@@ -400,7 +441,7 @@ project.ppm <- local({
             print(object.i)
           }
           # recurse
-          object.i <- project.ppm(object.i, fatal=fatal, trace=tdnext)
+          object.i <- emend.ppm(object.i, fatal=fatal, trace=tdnext)
           # evaluate logPL
           logPL.i   <- logLik(object.i, warn=FALSE)
           tracemessage(td, "max log pseudolikelihood = ", logPL.i)
@@ -474,7 +515,7 @@ project.ppm <- local({
         print(newobject)
       }
       # recurse
-      newobject <- project.ppm(newobject, fatal=fatal, trace=tdnext)
+      newobject <- emend.ppm(newobject, fatal=fatal, trace=tdnext)
       object <- newobject
     } else if(is.list(change) && all(unlist(lapply(change, is.interact)))) {
       # new style: 'project' returns a list of candidate interactions
@@ -501,7 +542,7 @@ project.ppm <- local({
           print(object.i)
         }
         # recurse
-        object.i <- project.ppm(object.i, fatal=fatal, trace=tdnext)
+        object.i <- emend.ppm(object.i, fatal=fatal, trace=tdnext)
         # evaluate logPL
         logPL.i   <- logLik(object.i, warn=FALSE)
         tracemessage(td, "max log pseudolikelihood = ", logPL.i)
@@ -521,19 +562,49 @@ project.ppm <- local({
     leaving(td)
     return(object)
   }
-  project.ppm
+  emend.ppm
 })
 
 # more methods
 
-logLik.ppm <- function(object, ..., new.coef=NULL, warn=TRUE) {
+deviance.ppm <- function(object, ...) {
+  satlogpl <- object$satlogpl
+  if(is.null(satlogpl)) {
+    object <- update(object, forcefit=TRUE)
+    satlogpl <- object$satlogpl
+  }
+  if(is.null(satlogpl) || !is.finite(satlogpl))
+    return(NA)
+  ll <- do.call(logLik,
+                resolve.defaults(list(object=object, absolute=FALSE),
+                                 list(...)))
+  ll <- as.numeric(ll)
+  2 * (satlogpl - ll)
+}
+
+logLik.ppm <- function(object, ..., new.coef=NULL, warn=TRUE, absolute=FALSE) {
   if(!is.poisson.ppm(object) && warn) 
-    warning(paste("log likelihood is not available for non-Poisson model;",
-                  "log-pseudolikelihood returned"))
+    warn.once("ppmLogLik",
+              "log likelihood is not available for non-Poisson model;",
+              "log pseudolikelihood returned")
+  ## degrees of freedom
+  nip <- if(!inherits(object, "ippm")) 0 else
+           length(attr(object$covfunargs, "free"))
+  df <- length(coef(object)) + nip
+  ## compute adjustment constant
+  if(absolute && object$method %in% c("exact", "mpl", "ho")) {
+    X <- data.ppm(object)
+    W <- Window(X)
+    areaW <-
+      if(object$correction == "border" && object$rbord > 0) 
+      eroded.areas(W, object$rbord) else area(W)
+    constant <- areaW * markspace.integral(X)
+  } else constant <- 0
+  ##
   if(is.null(new.coef)) {
     ## extract from object
-    ll <- object$maxlogpl
-    attr(ll, "df") <- length(coef(object))
+    ll <- object$maxlogpl + constant
+    attr(ll, "df") <- df
     class(ll) <- "logLik"
     return(ll)
   } 
@@ -552,7 +623,8 @@ logLik.ppm <- function(object, ..., new.coef=NULL, warn=TRUE) {
            w <- w.quad(Q)
            ll <- sum(log(cifdata[cifdata > 0])) - sum(w * cif)
          },
-         logi={
+         logi=,
+         VBlogi={
            B <- getglmdata(object, drop=TRUE)$.logi.B
            p <- cif/(B+cif)
            ll <- sum(log(p/(1-p))[Z]) + sum(log(1-p)) + sum(log(B[Z]))
@@ -560,11 +632,32 @@ logLik.ppm <- function(object, ..., new.coef=NULL, warn=TRUE) {
          stop(paste("Internal error: unrecognised ppm method:",
                     dQuote(method)))
          )
-  attr(ll, "df") <- length(coef(object))
+  ll <- ll + constant
+  attr(ll, "df") <- df
   class(ll) <- "logLik"
   return(ll)
 }
 
+pseudoR2 <- function(object, ...) {
+  UseMethod("pseudoR2")
+}
+
+pseudoR2.ppm <- function(object, ..., keepoffset=TRUE) {
+  dres <- deviance(object, ..., warn=FALSE)
+  nullfmla <- . ~ 1
+  if(keepoffset && has.offset.term(object)) {
+    off <- attr(model.depends(object), "offset")
+    offterms <- row.names(off)[apply(off, 1, any)]
+    if(length(offterms)) {
+      nullrhs <- paste(offterms, collapse=" + ") 
+      nullfmla <- as.formula(paste(". ~ ", nullrhs))
+    }
+  } 
+  nullmod <- update(object, nullfmla, forcefit=TRUE)
+  dnul <- deviance(nullmod, warn=FALSE)
+  return(1 - dres/dnul)
+}
+
 formula.ppm <- function(x, ...) {
   return(x$trend)
 }
@@ -594,11 +687,27 @@ labels.ppm <- function(object, ...) {
   return(lab[okterms])
 }
 
-extractAIC.ppm <- function (fit, scale = 0, k = 2, ...)
+AIC.ppm <- function(object, ..., k=2, takeuchi=TRUE) {
+  ll <- logLik(object, warn=FALSE)
+  pen <- attr(ll, "df")
+  if(takeuchi && !is.poisson(object)) {
+    vv <- vcov(object, what="internals")
+    logi <- (object$method == "logi")
+    J  <- with(vv, if(!logi) Sigma else (Sigma1log+Sigma2log))
+    H  <- with(vv, if(!logi) A1 else Slog)
+    ## Takeuchi penalty = trace of J H^{-1} = trace of H^{-1} J
+    JiH <- try(solve(H, J), silent=TRUE)
+    if(!inherits(JiH, "try-error")) 
+      pen <- sum(diag(JiH))
+  } 
+  return(- 2 * as.numeric(ll) + k * pen)
+}
+
+extractAIC.ppm <- function (fit, scale = 0, k = 2, ..., takeuchi=TRUE)
 {
-    edf <- length(coef(fit))
-    aic <- AIC(fit)
-    c(edf, aic + (k - 2) * edf)
+  edf <- length(coef(fit))
+  aic <- AIC(fit, k=k, takeuchi=takeuchi)
+  c(edf, aic)
 }
 
 #
@@ -612,17 +721,107 @@ model.frame.ppm <- function(formula, ...) {
     object <- update(object, forcefit=TRUE)
     gf <- getglmfit(object)
   }
-#  gd <- getglmdata(object)
-#  model.frame(gf, data=gd, ...)
-  model.frame(gf, ...)
+  argh <- resolve.defaults(list(formula=gf),
+                           list(...),
+                           list(data = getglmdata(object),
+                                subset = TRUE))
+  result <- switch(object$fitter,
+                   gam = do.call(modelFrameGam, argh),
+                   do.call(model.frame, argh))
+  return(result)
+}
+
+#' a hacked version of model.frame.glm that works for gam objects (mgcv)
+modelFrameGam <- function(formula, ...) {
+  dots <- list(...)
+  nargs <- dots[match(c("data", "na.action", "subset"), names(dots), 
+                      0L)]
+  if (length(nargs) || is.null(formula$model)) {
+    fcall <- formula$call
+#    fcall$method <- "model.frame"
+    fcall[[1L]] <- quote(mgcv::gam)
+    fcall[names(nargs)] <- nargs
+    env <- environment(formula$terms)
+    if (is.null(env)) 
+      env <- parent.frame()
+    refut <- eval(fcall, env)
+    refut$model
+  } else formula$model
 }
 
 #
 # method for model.matrix
 
-model.matrix.ppm <- function(object, data=model.frame(object),
-                             ..., keepNA=TRUE) {
-  data.given <- !missing(data)
+model.matrix.ppm <- function(object,
+                             data=model.frame(object, na.action=NULL),
+                             ..., Q=NULL, keepNA=TRUE) {
+  if(missing(data)) data <- NULL
+  PPMmodelmatrix(object, data=data, ..., Q=Q, keepNA=keepNA)
+}
+
+model.matrix.ippm <- function(object,
+                              data=model.frame(object, na.action=NULL),
+                              ..., Q=NULL, keepNA=TRUE, irregular=FALSE) {
+  if(missing(data)) data <- NULL 
+  PPMmodelmatrix(object, data=data, ...,
+                 Q=Q, keepNA=keepNA, irregular=irregular)
+}
+
+PPMmodelmatrix <- function(object,
+                           data = NULL, 
+                           ...,
+                           subset, Q=NULL, keepNA=TRUE, irregular=FALSE,
+                           splitInf=FALSE) {
+  # handles ppm and ippm			      
+  data.given <- !is.null(data)
+  irregular <- irregular && inherits(object, "ippm") && !is.null(object$iScore)
+  if(splitInf && !data.given && is.null(Q)) {
+    #' force re-computation
+    Q <- quad.ppm(object)
+  }
+  if(!is.null(Q)) {
+    if(data.given) stop("Arguments Q and data are incompatible")
+    if(!inherits(Q, c("ppp", "quad")))
+      stop("Q should be a point pattern or quadrature scheme")
+    if(is.ppp(Q)) Q <- quad(Q, Q[FALSE])
+    ## construct Berman-Turner frame
+    needed <- c("trend", "interaction", "covariates", "covfunargs",
+                "correction", "rbord")
+    bt <- do.call(bt.frame,
+                  c(list(Q), object[needed], list(splitInf=splitInf)))
+    forbid <- bt$forbid
+    ## compute model matrix
+    mf <- model.frame(bt$fmla, bt$glmdata, ...)
+    mm <- model.matrix(bt$fmla, mf, ...)
+    ass <- attr(mm, "assign")
+    if(irregular) {
+      ## add irregular score components
+      U <- union.quad(Q)
+      mi <- sapply(object$iScore, do.call,
+                   args=append(list(x=U$x, y=U$y), object$covfunargs),
+                   envir=environment(terms(object)))
+      if(nrow(mi) != nrow(mm))
+        stop("Internal error: incorrect number of rows in iScore")
+      mm <- cbind(mm, mi)
+    }
+    ## subset
+    if(!missing(subset)) {
+      ok <- eval(substitute(subset), envir=bt$glmdata)
+      mm <- mm[ok, , drop=FALSE]
+      if(!is.null(forbid)) forbid <- forbid[ok]
+    }
+    ## remove NA's ?
+    if(!keepNA) {
+      ok <- complete.cases(mm)
+      mm <- mm[ok, , drop=FALSE]
+      if(!is.null(forbid)) forbid <- forbid[ok]
+    }
+    attr(mm, "assign") <- ass 
+    attr(mm, "-Inf") <- forbid
+    return(mm)
+  }
+
+  #' extract GLM fit 
   gf <- getglmfit(object)
   if(is.null(gf)) {
     warning("Model re-fitted with forcefit=TRUE")
@@ -631,27 +830,60 @@ model.matrix.ppm <- function(object, data=model.frame(object),
     if(is.null(gf))
       stop("internal error: unable to extract a glm fit")
   }
-  
+
   if(data.given) {
-    # new data. Must contain the Berman-Turner variables as well.
+    #' new data. Must contain the Berman-Turner variables as well.
     bt <- list(.mpl.Y=1, .mpl.W=1, .mpl.SUBSET=TRUE)
     if(any(forgot <- !(names(bt) %in% names(data)))) 
-      data <- do.call("cbind", append(list(data), bt[forgot]))
-    mm <- model.matrix(gf, data=data, ...)
+      data <- do.call(cbind, append(list(data), bt[forgot]))
+    mm <- model.matrix(gf, data=data, ..., subset=NULL)
+    ass <- attr(mm, "assign")
+    if(irregular) {
+      ## add irregular score components 
+      mi <- sapply(object$iScore, do.call,
+                   args=append(list(x=data$x, y=data$y), object$covfunargs),
+                   envir=environment(terms(object)))
+      if(nrow(mi) != nrow(mm))
+        stop("Internal error: incorrect number of rows in iScore")
+      mm <- cbind(mm, mi)
+      attr(mm, "assign") <- ass
+    }
+    if(inherits(gf, "gam")) 
+      attr(mm, "assign") <- gf$assign
     return(mm)
   }
 
-  if(!keepNA) {
+  scrambled <- object$scrambled %orifnull% FALSE
+  ## if TRUE, this object was produced by 'subfits' using jittered covariate
+
+  if(!keepNA && !irregular && !scrambled) {
     # extract model matrix of glm fit object
     # restricting to its 'subset' 
-    mm <- model.matrix(gf, data=data, ...)
+    mm <- model.matrix(gf, ...)
+    if(inherits(gf, "gam")) 
+      attr(mm, "assign") <- gf$assign
     return(mm)
   }
   
-  # extract model matrix for all cases
-  mm <- model.matrix(gf, data, ..., subset=NULL)
+  ## extract model matrix for all cases
+  gd <- getglmdata(object, drop=FALSE) 
+  if(!scrambled) {
+    ## 'gf' was fitted to correct data. Use internals.
+    mm <- model.matrix(gf, ..., subset=NULL, na.action=NULL)
+    ass <- attr(mm, "assign")
+  } else {
+    ## 'gf' was originally fitted using jittered data:
+    ## Use correct data given by 'gd'
+    ## Temporarily add scrambled data to avoid singular matrices etc
+    gds <- object$internal$glmdata.scrambled
+    gdplus <- rbind(gd, gds)
+    mm <- model.matrix(gf, ..., data=gdplus, subset=NULL, na.action=NULL)
+    ass <- attr(mm, "assign")
+    ## Now remove rows corresponding to scrambled data
+    mm <- mm[seq_len(nrow(gd)), , drop=FALSE]
+    attr(mm, "assign") <- ass
+  } 
   cn <- colnames(mm)
-  gd <- getglmdata(object, drop=FALSE)
   if(nrow(mm) != nrow(gd)) {
     # can occur if covariates include NA's or interaction is -Inf
     insubset <- getglmsubset(object)
@@ -662,9 +894,35 @@ model.matrix.ppm <- function(object, data=model.frame(object),
       mmplus[isna, ] <- NA
       mmplus[!isna, ] <- mm
       mm <- mmplus
+      attr(mm, "assign") <- ass
     } else 
     stop("internal error: model matrix does not match glm data frame")
   }
+  if(irregular) {
+    ## add irregular score components 
+    U <- union.quad(quad.ppm(object, drop=FALSE))
+    mi <- sapply(object$iScore, do.call,
+                 args=append(list(x=U$x, y=U$y), object$covfunargs),
+	  envir=environment(terms(object)))
+    if(nrow(mi) != nrow(mm))
+      stop("Internal error: incorrect number of rows in iScore")
+    mm <- cbind(mm, mi)
+    attr(mm, "assign") <- ass
+    cn <- c(cn, colnames(mi))
+  }
+  ## subset
+  if(!missing(subset)) {
+    ok <- eval(substitute(subset), envir=gd)
+    mm <- mm[ok, , drop=FALSE]
+    attr(mm, "assign") <- ass
+  }
+  ## remove NA's
+  if(!keepNA) {
+    mm <- mm[complete.cases(mm), , drop=FALSE]
+    attr(mm, "assign") <- ass
+  }
+  if(inherits(gf, "gam")) 
+    attr(mm, "assign") <- gf$assign
   colnames(mm) <- cn
   return(mm)
 }
@@ -675,28 +933,34 @@ model.images <- function(object, ...) {
 
 model.images.ppm <- function(object, W=as.owin(object), ...) {
   X <- data.ppm(object)
-  # make a quadscheme with a dummy point at every pixel
+#  irregular <- resolve.1.default(list(irregular=FALSE), list(...))
+  ## make a quadscheme with a dummy point at every pixel
   Q <- pixelquad(X, W)
-  # construct Berman-Turner frame
-  needed <- c("trend", "interaction", "covariates", "correction", "rbord")
-  bt <- do.call("bt.frame", append(list(Q), object[needed]))
-  # compute model matrix
-  mf <- model.frame(bt$fmla, bt$glmdata, ...)
-  mm <- model.matrix(bt$fmla, mf, ...)
-  # retain only the entries for dummy points (pixels)
+  ## compute model matrix
+  mm <- model.matrix(object, Q=Q, ...)
+  ## retain only the entries for dummy points (pixels)
   mm <- mm[!is.data(Q), , drop=FALSE]
-  # create template image
+  mm <- as.data.frame(mm)
+  ## create template image
   Z <- as.im(attr(Q, "M"))
-  # make images
+  ok <- !is.na(Z$v)
+  ## make images
   imagenames <- colnames(mm)
-  result <- lapply(imagenames,
-                   function(nama, Z, mm) {
-                     Z$v[!is.na(Z$v)] <- mm[, nama]
-                     return(Z)
-                   },
-                   Z=Z, mm=mm)
-  result <- as.listof(result)
-  names(result) <- imagenames
+  if(!is.multitype(object)) {
+    result <- lapply(as.list(mm), replace, list=ok, x=Z)
+    result <- as.solist(result)
+    names(result) <- imagenames
+  } else {
+    marx <- marks(Q$dummy)
+    mmsplit <- split(mm, marx)
+    result <- vector(mode="list", length=length(mmsplit))
+    for(i in seq_along(mmsplit))
+      result[[i]] <- as.solist(lapply(as.list(mmsplit[[i]]),
+                                      replace, list=ok, x=Z))
+    names(result) <- names(mmsplit)
+    result <- do.call(hyperframe, result)
+    row.names(result) <- imagenames
+  }
   return(result)
 }
 
@@ -727,7 +991,7 @@ as.ppm.ppm <- function(object) {
   object
 }
 
-# method for as.owin
+## method for as.owin
 
 as.owin.ppm <- function(W, ..., from=c("points", "covariates"), fatal=TRUE) {
   if(!verifyclass(W, "ppm", fatal=fatal))
@@ -741,8 +1005,24 @@ as.owin.ppm <- function(W, ..., from=c("points", "covariates"), fatal=TRUE) {
   if(!any(isim))
     return(datawin)
   cwins <- lapply(covs[isim], as.owin)
-  covwin <- do.call("intersect.owin", unname(cwins))
+  covwin <- do.call(intersect.owin, unname(cwins))
   result <- intersect.owin(covwin, datawin)
   return(result)
 }
 
+domain.ppm <- Window.ppm <- function(X, ..., from=c("points", "covariates")) {
+  from <- match.arg(from)
+  as.owin(X, ..., from=from)
+}
+
+## change the coefficients in a ppm or other model
+
+tweak.coefs <- function(model, new.coef) {
+  if(is.null(new.coef)) return(model)
+  co <- coef(model)
+  check.nvector(new.coef, length(co), things="coefficients")
+  model$coef.orig <- co
+  model$coef <- new.coef
+  return(model)
+}
+
