@@ -4,7 +4,7 @@
 #	A class 'ppp' to define point patterns
 #	observed in arbitrary windows in two dimensions.
 #
-#	$Revision: 4.111 $	$Date: 2017/06/05 10:31:58 $
+#	$Revision: 4.113 $	$Date: 2020/05/03 03:10:05 $
 #
 #	A point pattern contains the following entries:	
 #
@@ -320,6 +320,8 @@ cobble.xy <- function(x, y, f=ripras, fatal=TRUE, ...) {
             # i is a window
             window <- i
             if(clip) window <- intersect.owin(window, x$window)
+            if(is.vanilla(unitname(window)))
+              unitname(window) <- unitname(x)
             ok <- inside.owin(x$x, x$y, window)
             x <- ppp(x$x[ok], x$y[ok], window=window, #SIC
                      marks=marksubset(x$marks, ok),
@@ -356,17 +358,22 @@ cobble.xy <- function(x, y, f=ripras, fatal=TRUE, ...) {
           x <- x[j]   # invokes code above
 
         if(drop) {
+          #' drop unused factor levels
           mx <- x$marks
           switch(markformat(mx),
                  none = { },
                  vector = {
                    if(is.factor(mx))
-                     marks(x) <- factor(mx)
+                     marks(x) <- factor(mx) # this preserves order of levels
                  },
                  dataframe = {
-                   isfac <- sapply(mx, is.factor)
-                   if(any(isfac))
-                     mx[, isfac] <- lapply(mx[, isfac], factor)
+                   #' must be an actual data frame, not a matrix
+                   if(is.data.frame(mx)) {
+                     ml <- as.list(mx)
+                     isfac <- sapply(ml, is.factor)
+                     if(any(isfac))
+                       mx[, isfac] <- as.data.frame(lapply(ml[isfac], factor))
+                   }
                  },
                  hyperframe = { })
         }
@@ -684,3 +691,44 @@ domain.ppp <- Window.ppp <- function(X, ...) { as.owin(X) }
   return(X)
 }
 
+#' convert any appropriate subset index for any kind of point pattern
+#' to a logical vector
+
+ppsubset <- function(X, I, Iname, fatal=FALSE) {
+  if(missing(Iname))
+    Iname <- deparse(substitute(I))
+  # I could be a window or logical image
+  if(is.im(I))
+    I <- solutionset(I)
+  if((is.ppp(X) || is.lpp(X)) && is.owin(I)) {
+    I <- inside.owin(X, w=I)
+    return(I)
+  }
+  if((is.pp3(X) && inherits(I, "box3")) ||
+     (is.ppx(X) && inherits(I, "boxx"))) {
+    I <- inside.boxx(X, w=I)
+    return(I)
+  }
+  # I could be a function to be applied to X
+  if(is.function(I)) {
+    I <- I(X)
+    if(!is.vector(I)) {
+      whinge <- paste("Function", sQuote(Iname), "did not return a vector")
+      if(fatal) stop(whinge, call.=FALSE)
+      warning(whinge, call.=FALSE)
+      return(NULL)
+    }
+  }      
+  # I is now a subset index: convert to logical
+  I <- grokIndexVector(I, npoints(X))$strict$lo
+
+  if(anyNA(I)) {
+    #' illegal entries
+    whinge <- paste("Indices in", sQuote(Iname), "exceed array limits")
+    if(fatal) stop(whinge, call.=FALSE)
+    warning(whinge, call.=FALSE)
+    return(NULL)
+  }
+
+  return(I)
+}
