@@ -4,7 +4,7 @@
 # computes residuals for fitted point process model
 #
 #
-# $Revision: 1.24 $ $Date: 2016/06/30 03:29:54 $
+# $Revision: 1.25 $ $Date: 2017/12/07 03:03:48 $
 #
 
 residuals.ppm <-
@@ -47,7 +47,7 @@ residuals.ppm <-
     } else {
       # estimate model parameters using a (presumably) denser set of dummy pts
       # Determine new quadrature scheme
-      if(inherits(quad, "quad")) 
+      if(is.quad(quad))
         hi.res.quad <- quad
       else if(is.ppp(quad))
         hi.res.quad <- quadscheme(data=data.ppm(object), dummy=quad)
@@ -88,7 +88,7 @@ residuals.ppm <-
     if(drop) {
       gs <- getglmsubset(object)
       ok <- !is.na(gs) & gs
-      X <- X[ok,]
+      X <- X[ok, , drop=FALSE]
     }
   }
       
@@ -98,7 +98,7 @@ residuals.ppm <-
                      raw     = rep.int(1, sum(Z)), 
                      inverse = 1/lambda[Z],
                      pearson = 1/sqrt(lambda[Z]),
-                     score   = X[Z, ]
+                     score   = X[Z, , drop=FALSE]
                      )
 
   density <- switch(type,
