@@ -3,7 +3,7 @@
 #
 #  Smooth the marks of a point pattern
 # 
-#  $Revision: 1.68 $  $Date: 2018/10/05 03:49:46 $
+#  $Revision: 1.74 $  $Date: 2020/02/04 01:55:03 $
 #
 
 # smooth.ppp <- function(X, ..., weights=rep(1, npoints(X)), at="pixels") {
@@ -56,18 +56,10 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
   }
 
   ## ensure weights are numeric
-  weightsgiven <- !missing(weights) && !is.null(weights) 
-  if(weightsgiven) {
-    # convert to numeric
-    if(is.im(weights)) {
-      weights <- safelookup(weights, X) # includes warning if NA
-    } else if(is.expression(weights)) 
-      weights <- eval(weights, envir=as.data.frame(X), enclos=parent.frame())
-    if(length(weights) == 0)
-      weightsgiven <- FALSE
-  }
-  if(weightsgiven) {
-    check.nvector(weights, npoints(X))
+  if(weightsgiven <- !missing(weights) && !is.null(weights)) {
+    pa <- parent.frame()
+    weights <- pointweights(X, weights=weights, parent=pa)
+    weightsgiven <- !is.null(weights)
   } else weights <- NULL
 
   ## geometric mean smoothing
@@ -148,7 +140,8 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
 
   if(diggle) {
     ## absorb Diggle edge correction into weights vector
-    edg <- second.moment.calc(X, sigma, what="edge", ...,                                                     varcov=varcov, adjust=adjust,
+    edg <- second.moment.calc(X, sigma, what="edge", ...,
+                              varcov=varcov, adjust=adjust,
                               kernel=kernel, scalekernel=scalekernel)
     ei <- safelookup(edg, X, warn=FALSE)
     weights <- if(weightsgiven) weights/ei else 1/ei
@@ -161,13 +154,13 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
 
   ## calculate...
   marx <- marks(X)
+  uhoh <- NULL
   if(!is.data.frame(marx)) {
     # ........ vector of marks ...................
     values <- marx
-    if(is.factor(values)) {
-      warning("Factor valued marks were converted to integers")
-      values <- as.numeric(values)
-    }
+    if(is.factor(values)) 
+      warning("Factor valued marks were converted to integers", call.=FALSE)
+    values <- as.numeric(values)
     ## detect constant values
     ra <- range(values, na.rm=TRUE)
     if(diff(ra) == 0) {
@@ -224,18 +217,24 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
                               | (denominator < eps))
                if(any(as.matrix(nbg), na.rm=TRUE)) {
                  warning(paste("Numerical underflow detected:",
-                               "sigma is probably too small"))
+                               "sigma is probably too small"),
+                         call.=FALSE)
+                 uhoh <- unique(c(uhoh, "underflow"))
                  ## l'Hopital's rule
                  distX <- distmap(X, xy=numerator)
                  whichnn <- attr(distX, "index")
                  nnvalues <- eval.im(values[whichnn])
                  result[nbg] <- nnvalues[nbg]
                }
-               attr(result, "warnings") <- attr(numerator, "warnings")
+               uhoh <- attr(numerator, "warnings")
              })
     }
   } else {
     ## ......... data frame of marks ..................
+    ## convert to numerical values
+    if(any(sapply(as.list(marx), is.factor)))
+      warning("Factor columns of marks were converted to integers", call.=FALSE)
+    marx <- asNumericMatrix(marx)
     ## detect constant columns
     ra <- apply(marx, 2, range, na.rm=TRUE)
     isconst <- (apply(ra, 2, diff) == 0)
@@ -275,8 +274,11 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
              points={
                if(is.null(uhoh)) {
                  ## numerators is a matrix (or may have dropped to vector)
-                 if(!is.matrix(numerators))
-                   numerators <- matrix(numerators, ncol=1)
+                 if(is.data.frame(numerators)) {
+                   numerators <- as.matrix(numerators)
+                 } else if(!is.matrix(numerators)) {
+                   numerators <- matrix(unlist(numerators), nrow=npoints(X))
+                 }
                  ratio <- numerators/denominator
                  if(any(badpoints <- matrowany(!is.finite(ratio)))) {
                    whichnnX <- nnwhich(X)
@@ -294,21 +296,29 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
                ## numerators is a list of images (or may have dropped to 'im')
                if(is.im(numerators))
                  numerators <- list(numerators)
-               ratio <- lapply(numerators, "/", e2=denominator)
-               if(!is.null(uhoh)) {
+               result <- solapply(numerators, "/", e2=denominator)
+               eps <- .Machine$double.eps
+               denOK <- eval.im(denominator >= eps)
+               if(!is.null(uhoh) || !all(denOK)) {
                  ## compute nearest neighbour map on same raster
                  distX <- distmap(X, xy=denominator)
                  whichnnX <- attr(distX, "index")
                  ## fix images
-                 for(j in 1:length(ratio)) {
-                   ratj <- ratio[[j]]
+                 allgood <- TRUE
+                 for(j in 1:length(result)) {
+                   ratj <- result[[j]]
                    valj <- marx[,j]
-                   ratio[[j]] <-
-                     eval.im(ifelseXY(is.finite(ratj), ratj, valj[whichnnX]))
+                   goodj <- eval.im(is.finite(ratj) & denOK)
+                   result[[j]] <- eval.im(goodj, ratj, valj[whichnnX])
+                   allgood <- allgood && all(goodj)
+                 }
+                 if(!allgood) {
+                   warning(paste("Numerical underflow detected:",
+                                 "sigma is probably too small"),
+                           call.=FALSE)
+                   uhoh <- unique(c(uhoh, "underflow"))
                  }
-                 attr(ratio, "warnings") <- uhoh
                }
-               result <- as.solist(ratio)
                names(result) <- colnames(marx)
              })
     } else result <- NULL 
@@ -338,10 +348,9 @@ Smooth.ppp <- function(X, sigma=NULL, ...,
     }
   }
   ## wrap up
-  attr(result, "warnings") <-
-    unlist(lapply(result, attr, which="warnings"))
   attr(result, "sigma") <- sigma
   attr(result, "varcov") <- varcov
+  attr(result, "warnings") <- uhoh
   return(result)
 }
 
@@ -706,21 +715,12 @@ bw.smoothppp <- function(X, nh=spatstat.options("n.bandwidth"),
   }
 
   # optimize
-  iopt <- which.min(cv)
-#  hopt <- h[iopt]
-  #
-  if(warn && (iopt == nh || iopt == 1)) 
-    warning(paste("Cross-validation criterion was minimised at",
-                  if(iopt == 1) "left-hand" else "right-hand",
-                  "end of interval",
-                  paste(prange(signif(c(hmin, hmax), 3)), ";", sep=""),
-                  "use arguments hmin, hmax to specify a wider interval"),
-            call.=FALSE)
-  #
-  result <- bw.optim(cv, h, iopt,
+  result <- bw.optim(cv, h, 
                      hname="sigma",
                      creator="bw.smoothppp",
                      criterion="Least Squares Cross-Validation",
+                     warnextreme=warn,
+                     hargnames=c("hmin", "hmax"),
                      unitname=unitname(X))
   return(result)
 }
@@ -826,6 +826,7 @@ smoothcrossEngine <- function(Xdata, Xquery, values, sigma, ...,
       numerator <- tapplysum(contribij, list(jfac))
       result <- numerator/denominator
     } else {
+      result <- matrix(, nrow=nquery, ncol=k)
       for(kk in 1:k) {
         contribij <- wkerij * values[i, kk]
         numeratorkk <- tapplysum(contribij, list(jfac))
