@@ -1,20 +1,22 @@
 #
 # split.ppp.R
 #
-# $Revision: 1.22 $ $Date: 2014/04/21 10:45:34 $
+# $Revision: 1.36 $ $Date: 2020/05/08 01:22:18 $
 #
 # split.ppp and "split<-.ppp"
 #
 #########################################
 
-split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
+split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, reduce=FALSE, ...) {
   verifyclass(x, "ppp")
   mf <- markformat(x)
+  fgiven <- !missing(f)
   
-  if(is.null(un))
-    un <- missing(f) && (mf != "dataframe")
+  if(is.null(un)) {
+    un <- !fgiven && (mf != "dataframe")
+  } else un <- as.logical(un)
 
-  if(missing(f)) {
+  if(!fgiven) {
     # f defaults to marks of x
     switch(mf,
            none={
@@ -31,17 +33,27 @@ split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
                stop("Data frame of marks contains no factors")
            })
     splittype <- "factor"
-  } else{
+  } else {
     # f was given
     fsplit <- f
     if(is.factor(f)) {
       splittype <- "factor"
+    } else if(is.logical(f)) {
+      splittype <- "factor"
+      f <- factor(f)
     } else if(is.tess(f)) {
       # f is a tessellation: determine the grouping
       f <- marks(cut(x, fsplit))
       splittype <- "tess"
+    } else if(is.owin(f)) {
+      # f is a window: coerce to a tessellation
+      W <- as.owin(x)
+      fsplit <- tess(tiles=list(fsplit, setminus.owin(W, fsplit)),
+                     window=W)
+      f <- marks(cut(x, fsplit))
+      splittype <- "tess"
     } else if(is.im(f)) {
-      # f is an image: determine the grouping
+      # f is an image: coerce to a tessellation
       fsplit <- tess(image=f)
       f <- marks(cut(x, fsplit))
       splittype <- "tess"
@@ -55,7 +67,8 @@ split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
       splittype <- "factor"
     } else 
       stop(paste("f must be",
-                 "a factor, a tessellation, an image,",
+                 "a factor, a logical vector,", 
+                 "a tessellation, a window, an image,",
                  "or the name of a column of marks"))
     if(length(f) != npoints(x))
       stop("length(f) must equal the number of points in x")
@@ -83,30 +96,47 @@ split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
            stop("Internal error: wrong format for fsplit"))
   }
 
-  # split the data
-  out <- list()
-  for(l in lev) 
-    out[[paste(l)]] <- x[!is.na(f) & (f == l)]
+  ## remove marks that will not be retained
+  if(un && reduce && mf == "dataframe")
+    warning("Incompatible arguments un=TRUE and reduce=TRUE: assumed un=TRUE")
+  if(un) {
+    x <- unmark(x)
+  } else if(reduce && !fgiven && mf == "dataframe") {
+    # remove the column of marks that determined the split
+    j <- findfirstfactor(marks(x))
+    if(!is.null(j))
+      marks(x) <- marks(x)[, -j]
+  }
   
-  if(un)
-     out <- lapply(out, unmark)
+  ## split the data
+  out <- list()
+  fok <- !is.na(f)
+  for(l in lev) 
+    out[[paste(l)]] <- x[fok & (f == l)]
+
+  ## 
   if(splittype == "tess") {
     til <- tiles(fsplit)
     for(i in seq_along(out))
       out[[i]]$window <- til[[i]]
   }
-  class(out) <- c("splitppp", class(out))
+  class(out) <- c("splitppp", "ppplist", "solist", class(out))
   attr(out, "fsplit") <- fsplit
+  attr(out, "fgroup") <- f
   return(out)
 }
 
-"split<-.ppp" <- function(x, f=marks(x), drop=FALSE, un=missing(f), 
+"split<-.ppp" <- function(x, f=marks(x), drop=FALSE, un=NULL,
                           ..., value) {
   verifyclass(x, "ppp")
   W <- x$window
+  fgiven <- !missing(f)
   mf <- markformat(x)
+  
   # evaluate `un' before assigning value of 'f'
-  force(un)
+  if(is.null(un)) {
+    un <- !fgiven && (mf != "dataframe")
+  } else un <- as.logical(un)
 
   # validate assignment value
   stopifnot(is.list(value))
@@ -155,9 +185,11 @@ split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
         fsplit <- f <- marx[[f]]
       else
         stop(paste("The name", sQuote(f), "does not match any column of marks"))
+    } else if(is.logical(f)) {
+      f <- factor(f)
     } else if(!is.factor(f))
       stop(paste("f must be",
-                 "a factor, a tessellation, an image,",
+                 "a factor, a logical vector, a tessellation, an image,",
                  "or the name of a column of marks"))
     if(length(f) != x$n)
       stop("length(f) must equal the number of points in x")
@@ -202,7 +234,16 @@ split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
   }
 
   # put Humpty together again
-  out <- do.call(superimpose,c(value,list(W=W)))
+  if(npoints(x) == length(f) &&
+     length(levels(f)) == length(value) &&
+     all(table(f) == sapply(value, npoints))) {
+    ## exact correspondence
+    out <- x
+    for(i in seq_along(levels(f)))
+      out[ f == lev[i] ] <- value[[i]]
+  } else {
+    out <- do.call(superimpose,c(value,list(W=W)))
+  }
   return(out)
 }
 
@@ -210,7 +251,8 @@ split.ppp <- function(x, f = marks(x), drop=FALSE, un=NULL, ...) {
 print.splitppp <- function(x, ...) {
   f <- attr(x, "fsplit")
   what <- if(is.tess(f)) "tessellation" else
-          if(is.factor(f)) "factor" else "unknown data"
+          if(is.factor(f)) "factor" else 
+          if(is.logical(f)) "logical vector" else typeof(f)
   cat(paste("Point pattern split by", what, "\n"))
   nam <- names(x)
   for(i in seq_along(x)) {
@@ -227,7 +269,7 @@ summary.splitppp <- function(object, ...) {
 }
 
 print.summary.splitppp <- function(x, ...) {
-  class(x) <- "listof"
+  class(x) <- "anylist"
   print(x)
   invisible(NULL)
 }
@@ -270,14 +312,29 @@ print.summary.splitppp <- function(x, ...) {
   x
 }
   
-density.splitppp <- function(x, ...) {
-  as.listof(lapply(x, density, ...))
+density.ppplist <- 
+density.splitppp <- function(x, ..., weights=NULL, se=FALSE) {
+  if(is.null(weights) || is.im(weights) || is.expression(weights))
+    weights <- rep(list(weights), length(x))
+  y <- mapply(density.ppp, x=x, weights=weights,
+              MoreArgs=list(se=se, ...),
+              SIMPLIFY=FALSE)
+  if(!se) return(as.solist(y, demote=TRUE))
+  y.est <- lapply(y, getElement, name="estimate")
+  y.se  <- lapply(y, getElement, name="SE")
+  z <- list(estimate = as.solist(y.est, demote=TRUE),
+            SE       = as.solist(y.se,  demote=TRUE))
+  return(z)
 }
 
+
 plot.splitppp <- function(x, ..., main) {
   if(missing(main)) main <- short.deparse(substitute(x))
-  do.call("plot.listof",
+  do.call(plot.solist,
           resolve.defaults(list(x=x, main=main),
                            list(...),
                            list(equal.scales=TRUE)))
 }
+
+as.layered.splitppp <- function(X) { do.call(layered, X) }
+
