@@ -1,6 +1,6 @@
 # superimpose.R
 #
-# $Revision: 1.36 $ $Date: 2016/10/17 06:48:25 $
+# $Revision: 1.37 $ $Date: 2017/12/30 05:03:15 $
 #
 #
 ############################# 
@@ -45,24 +45,16 @@ superimpose.ppp <- function(..., W=NULL, check=TRUE) {
 
   # determine whether there is any window information
   if(!is.owin(W)) {
-    # we have to compute the final window
-    WXY <- NULL
-    Wppp <- NULL
-    if(any(isppp <- unlist(lapply(arglist, is.ppp)))) {
-      # extract windows from ppp objects
-      wins <- unname(lapply(arglist[isppp], as.owin))
-      # take union
-      Wppp <- if(length(wins) == 1) wins[[1]] else do.call(union.owin, wins)
-    } 
+    ## we have to compute the final window
     if(is.function(W)) {
-      # W is a function like bounding.box.xy or ripras
-      # Apply function to the x,y coordinates; it should return an owin
+      ## W is a function like bounding.box.xy or ripras
+      ## Apply function to the x,y coordinates; it should return an owin
       WXY <- W(XY)
       if(!is.owin(WXY))
         stop("Function W did not return an owin object", call.=FALSE)
-    }
-    if(is.character(W)) {
-      # character string identifies a function
+      W <- WXY
+    } else if(is.character(W)) {
+      ## character string identifies a function
       pW <- pmatch(W, c("convex", "rectangle", "bbox", "none"))
       if(is.na(pW))
         stop(paste("Unrecognised option W=", sQuote(W)), call.=FALSE)
@@ -73,12 +65,18 @@ superimpose.ppp <- function(..., W=NULL, check=TRUE) {
                     none=NULL)
       # in these cases we don't need to verify that the points are inside.
       needcheck <- !is.null(WXY)
-    }
-    if(is.null(WXY) && is.null(Wppp)) {
-      # no window information
-      return(XY)
-    }
-    W <- union.owin(WXY, Wppp)
+      if(!is.null(WXY)) W <- WXY
+    } else if(is.null(W)) {
+      if(any(isppp <- unlist(lapply(arglist, is.ppp)))) {
+        ## extract windows from ppp objects
+        wins <- unname(lapply(arglist[isppp], as.owin))
+        ## take union
+        W <- if(length(wins) == 1) wins[[1]] else do.call(union.owin, wins)
+      } else {
+        ## no window information
+        return(XY)
+      }
+    } else stop("Argument W is not understood")
   }
   # extract the marks if any
   nobj <- lengths(lapply(arglist, getElement, name="x"))
@@ -177,96 +175,3 @@ superimposeMarks <- function(arglist, nobj) {
   return(marx)
 }
 
-#==+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===+===
-
-  # This function is now deprecated.
-superimposePSP <-
-  function(..., W=NULL, check=TRUE)
-{
-  .Deprecated("superimpose","spatstat")
-  
-  # superimpose any number of line segment patterns
-  arglist <- list(...)
-
-  nargue <- length(arglist)
-  if(nargue == 0)
-    stop("No line segment patterns given", call.=FALSE)
-  
-  # catch possible abuses
-  if(is.null(W) && any(suspicious <- (names(arglist) == "window"))) {
-    id <- min(which(suspicious))
-    Win <- arglist[[id]]
-    if(is.owin(Win) || is.null(Win)) {
-      W <- Win
-      arglist <- arglist[-id]
-      nargue <- length(arglist)
-    }
-  }
-
-  # unpack a list
-  if(nargue == 1) {
-    X <- arglist[[1]]
-    if(!inherits(X, "psp") && inherits(X, "list"))
-      arglist <- X
-  }
-
-  isnull <- unlist(lapply(arglist, is.null))
-  arglist <- arglist[!isnull]
-  
-  if(!all(unlist(lapply(arglist, is.psp))))
-    stop("Some of the arguments are not psp objects", call.=FALSE)
-  
-  # extract segment coordinates
-  matlist <- lapply(arglist, function(x) { as.matrix(x$ends) })
-  # tack them together
-  mat <- do.call(rbind, matlist)
-
-  # extract marks if any
-  marxlist <- lapply(arglist, marks)
-
-  # check on compatibility of marks
-  mkfmt <- sapply(marxlist,markformat)
-  if(length(unique(mkfmt))>1)
-	stop(paste("The marks of the point patterns have different formats:",
-                   commasep(sQuote(mkfmt))),
-             call.=FALSE)
-  mkfmt <- mkfmt[1]
-  if(mkfmt=="dataframe") {
-	mcnms <- lapply(marxlist,names)
-	cdim  <- lengths(mcnms)
-	OK    <- length(unique(cdim)) == 1
-	if(OK) {
-		allInOne <- sapply(mcnms,paste,collapse="")
-		OK <- length(unique(allInOne)) == 1
-		if(!OK)
-                  stop("Data frames of marks have different names", call.=FALSE)
-	} else stop("Data frames of marks have different column dimensions",
-                    call.=FALSE)
-  }
- 
-  # combine the marks
-  marx <- switch(mkfmt,
-                 none = NULL,
-                 vector = {
-                   marxlist <- lapply(marxlist,
-                                      as.data.frame.vector,
-                                      nm="v1")
-                   do.call(rbind, marxlist)[,1]
-                 },
-                 dataframe = do.call(rbind, marxlist))
-
-  # determine window
-  if(!is.null(W))
-    W <- as.owin(W)
-  else {
-    # extract windows from psp objects
-    Wlist <- lapply(arglist, as.owin)
-    # take the union of all the windows
-    W <- NULL
-    for(i in seq_along(Wlist))
-      W <- union.owin(W, Wlist[[i]])
-  }
-
-  return(as.psp(mat, window=W, marks=marx, check=check))
-}
-
