@@ -10,6 +10,7 @@
 
   Macros used: FNAME = function name,
                WEIGHTED = #defined for weighted version
+               DISTINCT = #defined if contributions with i=j are omitted
 */
 
 void FNAME(
@@ -37,7 +38,31 @@ void FNAME(
   OUTERCHUNKLOOP(i, N, maxchunk, 256) {
     R_CheckUserInterrupt();
     INNERCHUNKLOOP(i, N, maxchunk, 256) {
-      /* loop over j != i */
+#ifndef DISTINCT
+      /* ...........  loop over j .......................... */
+      for(j = 0; j < N; j++) {
+	/* pointers to [i,j] and [j,i] in N*N matrices */
+	ijpos = i + N * j;
+	jipos = j + N * i;
+	/* pointers to x[, i, j] and x[ , j, i] */
+	xij = x + ijpos * P;
+	xji = x + jipos * P;
+	/* outer product */ 
+#ifdef WEIGHTED
+	wij = w[ijpos];
+#endif
+	for(k = 0; k < P; k++) {
+	  for(m = 0; m < P; m++) {
+#ifdef WEIGHTED
+	    y[m + k * P] += wij * xij[m] * xji[k];
+#else
+	    y[m + k * P] += xij[m] * xji[k];
+#endif
+	  }
+	}
+      }
+#else      
+      /* ............  loop over j != i .......................*/
       if(i > 0) {
 	for(j = 0; j < i; j++) {
 	  /* pointers to [i,j] and [j,i] in N*N matrices */
@@ -85,6 +110,7 @@ void FNAME(
 	}
       }
       /* end of loop over j */
+#endif      
     }
   }
 }
